/*  tif2hp.c  send 300 dpi tif file to the laserjet */
/*  note use of IO cntrl function to send the raster data to
    the printer--- */
/* 1/4/90  modified to also read mac tifs  */

#include "stdio.h"
#include "math.h"
#include <fcntl.h>
#include "sys\types.h"
#include "sys\stat.h"
#include "io.h"
#include "dos.h"

char buffer[8000];
FILE *printer;
FILE *pinfile;
int printer2;
char infile[40];
int reverse=0;

struct header{
	int byteorder;
	int version;
       long firstoffset;
      } header0;

struct directory{
	int tag;
	int type;
       long length;
       long count;
       } directory0;

struct header *pheader;
struct directory *pdir;
int bytelength;
int bytewidth;
int startofdata;


main(argc,argv)
int argc;
char *argv[];
{

int bytesread;
int filewidth;
int filelength;
int extra;

	  if(argc <2){
	    printf("correct usage tif2hp  infile1 \n");
	    perror("");
	    exit(0);
	   }


	strcpy(infile,argv[1]);

    if ((pinfile = fopen(infile,"rb")) == NULL){
	printf("unable to open %s\n",infile);
	perror("");
	exit(0);
	}

	bytesread = fread(buffer,1,300,pinfile);
	   if (bytesread != 300) {
		perror("did not read 300 bytes\n");
		exit(0);
	   }


	 getinfo(&filewidth,&filelength,&startofdata);

	   extra =filewidth % 8;
	   if (extra)
	    bytewidth = (filewidth / 8)+1;
	else
	    bytewidth = filewidth /8;


	   extra =filelength % 8;
	   if (extra)
	    bytelength = (filelength / 8)+1;
	else
	    bytelength = filelength /8;





	 initialize_printer();
	 send_data();
	 close(printer2);
	 fclose(printer);

	}

initialize_printer()
{
  printer = fopen("LPT1","w");
  if (printer == NULL)
	printf("error in opening the printer\n");

}

printerstream()
{
 /* second printer stream in binary format to send raster data */
union REGS inregs;
union REGS outregs;

 printer2 =open("LPT1",O_WRONLY | O_CREAT | O_BINARY,S_IWRITE);   /* second printer han*/

	    if(printer2 ==-1){
	      perror("unable to open the printer in this mode");
	      exit(0);
	      }

      /*------------------------------------------*/
      /* prepare for iocntl change of printer2 to binary */
     inregs.x.ax=0x4401;    /* call 44h set */
     inregs.x.bx= printer2;
     inregs.x.dx=0x00e0;   /* set to binary */

     intdos(&inregs,&outregs);

}

send_data()
{
 int x,y;
 int byteswritten;
 int bytesread;
 int bytesperread;
 char *pbuffer;
 int lengthofdata;
 int numofrows;

/*   static char bytestring[]={27,'*','b','7','5','W'};  */
 static char bytestring[10];
 static char closestring[]={27,'*','r','B'};
 static char endstring[]={12,27,'E'};



   sprintf(bytestring,"%c%c%c%d%c",27,'*','b',bytewidth,'W');
   lengthofdata = strlen(bytestring);

      fprintf(printer,"%c%s",27,"*p0x0Y");
      fprintf(printer,"%c%s",27,"*t300R");   /* raster graphics to 300 dpi */
      fprintf(printer,"%c%s",27,"*r1A");     /*  left margin at currect cursor */
      fflush(printer);	  /* make sure all data from this string written */
      printerstream();		 /* binary format for steam */


      pbuffer = buffer;
      bytesperread = (8000 / bytewidth)*bytewidth;

      fseek(pinfile,(long)startofdata,SEEK_SET);
      bytesread = fread(buffer,1,bytesperread,pinfile);

     if( ! reverse)		     /* mac file seems not reversed */
      for(y=0;y<bytesread;y++)
       *pbuffer = ~ *(pbuffer++);   /* not for laser jet */

      pbuffer = buffer;
      numofrows = bytesread / bytewidth;

      while(bytesread){
	  for(y=0;y<numofrows;y++) {
   byteswritten= write(printer2,bytestring,lengthofdata); /*data header  */
   byteswritten= write(printer2,pbuffer,bytewidth);
	  pbuffer+=bytewidth;
	  }
	 pbuffer=buffer;
      bytesread = fread(buffer,1,bytesperread,pinfile);

     if(! reverse)
      for(y=0;y<bytesread;y++)
       *pbuffer = ~ *(pbuffer++);   /* not for laser jet */

      pbuffer = buffer;
      numofrows = bytesread / bytewidth;
      }

    fprintf(printer,"%c%s",27,"*rB");
    fprintf(printer,"%c%c%c",12,27,'E');
}


getinfo(width,length,startofdata)
int *width;
int *length;
long *startofdata;
{
int x;
int tag100;
int *pint;
int numofdir;
int bytesread;
char temp;

	pheader = buffer;

	if(pheader->byteorder == 0x4D4D){
	   reverse=1;
	   swapheader(pheader);
	 }

	fseek(pinfile,pheader->firstoffset,SEEK_SET);
	bytesread = fread(buffer,1,1024,pinfile);

	   if (bytesread == 0) {
		perror("could not read the file\n");
		exit(0);
	   }

	if (reverse){
		   temp = buffer[0];
		   buffer[0]=buffer[1];
		   buffer[1]=temp;
		  }
	pint = (int *) &buffer[0];
	numofdir = *pint;
	pdir = buffer+2;

	if(reverse)
	  swapbytes(pdir,numofdir);

	 for (x=0; x < numofdir;x++){

	     if(pdir->type == 3)
		if(reverse)
		pdir->count = pdir->count >>16;  /* int type */

		     if(pdir->tag == 0x100){
				       *width = (int ) pdir->count;
				       tag100 =(int) (( char *)pdir - buffer);
					    }

		     if(pdir->tag == 0x101){
					  *length = (int ) pdir->count;
					    }

		     if(pdir->tag == 0x103){
				   if( pdir->count != 1){
				      printf("cannot use a compressed file \n");
				      exit(0);
				    }
		       }

		     if(pdir->tag == 0x111){
					  *startofdata =  pdir->count;
					    }

	  pdir++;
	 }
}

swapheader(pheader)
struct header *pheader;
{

struct header *pcopy;
unsigned char buffer[20];
unsigned char *pbuffer;
int x;
int limit;

	limit = sizeof(*pheader);
	pbuffer = (unsigned char *) pheader;

	 for(x=0;x<limit;x++){
	     buffer[x] = *(pbuffer++);
	}
	pbuffer = (unsigned char *) pheader;

       *(pbuffer++) = buffer[1];
       *(pbuffer++) = buffer[0];
       *(pbuffer++) = buffer[3];
       *(pbuffer++) = buffer[2];

       *(pbuffer++) = buffer[7];
       *(pbuffer++) = buffer[6];
       *(pbuffer++) = buffer[5];
       *(pbuffer++) = buffer[4];

}

swapbytes(pdir,numofdir)
struct directory *pdir;
int numofdir;
{

struct directory *pcopy;
unsigned char buffer[20];
unsigned char *pbuffer;
int x,xx;
int limit;
int start;

	limit = sizeof(*pdir);
	pbuffer = (unsigned char *) pdir;

	 for(x=0;x<limit;x++){
	     buffer[x] = *(pbuffer++);
	}

	pbuffer = (unsigned char *) pdir;

    for(xx=0;xx<numofdir;xx++){
       *(pbuffer++) = buffer[1];
       *(pbuffer++) = buffer[0];
       *(pbuffer++) = buffer[3];
       *(pbuffer++) = buffer[2];

       *(pbuffer++) = buffer[7];
       *(pbuffer++) = buffer[6];
       *(pbuffer++) = buffer[5];
       *(pbuffer++) = buffer[4];

       *(pbuffer++) = buffer[11];
       *(pbuffer++) = buffer[10];
       *(pbuffer++) = buffer[9];
       *(pbuffer++) = buffer[8];

    pdir++;

	pbuffer = (unsigned char *) pdir;

	 for(x=0;x<limit;x++){
	     buffer[x] = *(pbuffer++);
	}

	pbuffer = (unsigned char *) pdir;
    }
}
